==> mule-module-extensions-support-4.11.0-20251015.194132-110-test-sources.jar: org/mule/runtime/module/extension/internal/runtime/resolver/NullSafeValueResolverWrapperTestCase.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.runtime.resolver;

import static org.mule.runtime.api.el.BindingContextUtils.getTargetBindingContext;
import static org.mule.runtime.api.message.Message.of;
import static org.mule.test.module.extension.internal.util.ExtensionsTestUtils.toMetadataType;

import static java.util.Optional.empty;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.Is.is;

import static org.mockito.Answers.RETURNS_DEEP_STUBS;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mock.Strictness.LENIENT;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import org.mule.metadata.api.model.MetadataType;
import org.mule.runtime.api.el.CompiledExpression;
import org.mule.runtime.api.exception.MuleException;
import org.mule.runtime.api.message.Message;
import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.api.transformation.TransformationService;
import org.mule.runtime.core.api.Injector;
import org.mule.runtime.core.api.MuleContext;
import org.mule.runtime.core.api.el.ExpressionManagerSession;
import org.mule.runtime.core.api.el.ExtendedExpressionManager;
import org.mule.runtime.core.api.event.CoreEvent;
import org.mule.runtime.extension.api.annotation.param.NullSafe;
import org.mule.runtime.extension.api.annotation.param.Optional;
import org.mule.runtime.extension.api.annotation.param.Parameter;
import org.mule.runtime.extension.api.runtime.parameter.Literal;
import org.mule.runtime.module.extension.api.runtime.resolver.ValueResolver;
import org.mule.runtime.module.extension.api.runtime.resolver.ValueResolvingContext;
import org.mule.runtime.module.extension.internal.util.ReflectionCache;
import org.mule.tck.junit4.AbstractMuleTestCase;
import org.mule.tck.junit5.InjectDataWeave;
import org.mule.tck.size.SmallTest;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import io.qameta.allure.Issue;

@SmallTest
@ExtendWith(MockitoExtension.class)
class NullSafeValueResolverWrapperTestCase extends AbstractMuleTestCase {

  @Mock(answer = RETURNS_DEEP_STUBS, strictness = LENIENT)
  private CoreEvent event;

  @Mock(strictness = LENIENT)
  private MuleContext muleContext;

  @Mock
  private ObjectTypeParametersResolver objectTypeParametersResolver;

  @Mock(strictness = LENIENT)
  private Injector injector;

  @Mock
  private TransformationService transformationService;

  @InjectDataWeave
  private ExtendedExpressionManager expressionManager;

  private final ReflectionCache reflectionCache = new ReflectionCache();

  @BeforeEach
  public void setUp() throws MuleException {
    when(event.getError()).thenReturn(empty());
    when(event.getAuthentication()).thenReturn(empty());
    Message msg = of(null);
    when(event.getMessage()).thenReturn(msg);
    when(event.asBindingContext()).thenReturn(getTargetBindingContext(msg));
    when(event.getItemSequenceInfo()).thenReturn(empty());

    when(muleContext.getInjector()).thenReturn(injector);
    when(injector.inject(any(TypeSafeExpressionValueResolver.class))).thenAnswer(inv -> {
      inv.getArgument(0, TypeSafeExpressionValueResolver.class).setTransformationService(transformationService);
      return null;
    });
  }

  @Test
  void testMapType() throws MuleException {
    assertExpected(new StaticValueResolver<>(null), toMetadataType(HashMap.class), false, new HashMap<>());
  }

  @Test
  void testPojoType() throws MuleException {
    ExpressionManagerSession session = mock(ExpressionManagerSession.class);
    when(session.evaluate(any(CompiledExpression.class), any(DataType.class)))
        .thenAnswer(inv -> new TypedValue<>(5, inv.getArgument(1)));
    expressionManager = spy(expressionManager);
    when(expressionManager.openSession(any())).thenReturn(session);

    assertExpected(new StaticValueResolver<>(null), toMetadataType(DynamicPojo.class), true, new DynamicPojo(5));

    verify(event, times(1)).asBindingContext();
  }

  @Test
  @Issue("W-14954976")
  void testPojoTypeWithLiteralExpression() throws MuleException {
    assertExpected(new StaticValueResolver<>(null),
                   toMetadataType(DynamicPojoWithLiteralParam.class),
                   false,
                   new DynamicPojoWithLiteralParam(new ImmutableLiteral<>("#[5]", Integer.class)));

    verify(event, times(1)).asBindingContext();
  }

  @Test
  void testPojoWithStaticDefaultValue() throws MuleException {
    when(transformationService.transform(eq("false"), any(), any())).thenReturn(false);
    when(muleContext.getTransformationService()).thenReturn(transformationService);
    assertExpected(new StaticValueResolver<>(null), toMetadataType(NonDynamicPojo.class), false, new NonDynamicPojo(false));
  }

  @Test
  void testPojoWithMap() throws MuleException {
    DynamicPojoWithMap pojo = new DynamicPojoWithMap();
    pojo.setMap(new HashMap<>());
    assertExpected(new StaticValueResolver<>(null), toMetadataType(DynamicPojoWithMap.class), false, pojo);
  }

  @Test
  void testNullSafeSdkAndLegacyAnnotation() throws MuleException {
    PojoUsingSdkApiAndLegacyApi pojo = new PojoUsingSdkApiAndLegacyApi();
    pojo.setParameters(new ASimplePojo(), new ASimplePojo());
    assertExpected(new StaticValueResolver<>(null), toMetadataType(PojoUsingSdkApiAndLegacyApi.class), false, pojo);
  }

  private void assertExpected(ValueResolver valueResolver, MetadataType type, boolean isDynamic, Object expected)
      throws MuleException {
    ValueResolver resolver = NullSafeValueResolverWrapper.of(valueResolver, type, reflectionCache, expressionManager,
                                                             muleContext, objectTypeParametersResolver);
    ValueResolvingContext ctx = ValueResolvingContext.builder(event)
        .withExpressionManager(expressionManager)
        .build();
    assertThat(resolver.isDynamic(), is(isDynamic));
    assertThat(resolver.resolve(ctx), is(expected));
  }

  public static class DynamicPojo {

    public DynamicPojo() {}

    public DynamicPojo(int time) {
      this.time = time;
    }

    @Parameter
    @Optional(defaultValue = "#[5]")
    private int time;

    public int getTime() {
      return time;
    }

    public void setTime(int time) {
      this.time = time;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojo) {
        DynamicPojo that = (DynamicPojo) o;
        return time == that.time;
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(time);
    }
  }


  public static class DynamicPojoWithLiteralParam {

    public DynamicPojoWithLiteralParam() {}

    public DynamicPojoWithLiteralParam(Literal<Integer> time) {
      this.time = time;
    }

    @Parameter
    @Optional(defaultValue = "#[5]")
    private Literal<Integer> time;

    public Literal<Integer> getTime() {
      return time;
    }

    public void setTime(Literal<Integer> time) {
      this.time = time;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojoWithLiteralParam) {
        DynamicPojoWithLiteralParam that = (DynamicPojoWithLiteralParam) o;
        return time.getLiteralValue().equals(that.time.getLiteralValue());
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(time.getLiteralValue());
    }
  }


  public static class DynamicPojoWithMap {

    public DynamicPojoWithMap() {}

    @Parameter
    @Optional
    @NullSafe
    private Map<String, String> map;

    public Map<String, String> getMap() {
      return map;
    }

    public void setMap(Map<String, String> map) {
      this.map = map;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojoWithMap) {
        DynamicPojoWithMap that = (DynamicPojoWithMap) o;
        return Objects.equals(map, that.map);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(map);
    }
  }


  public static class NonDynamicPojo {

    public NonDynamicPojo() {}

    public NonDynamicPojo(Boolean staticDefaultValue) {
      this.staticDefaultValue = staticDefaultValue;
    }

    @Parameter
    @Optional(defaultValue = "false")
    private Boolean staticDefaultValue;

    public Boolean getStaticDefaultValue() {
      return staticDefaultValue;
    }

    public void setStaticDefaultValue(Boolean staticDefaultValue) {
      this.staticDefaultValue = staticDefaultValue;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof NonDynamicPojo) {
        NonDynamicPojo that = (NonDynamicPojo) o;
        return Objects.equals(staticDefaultValue, that.staticDefaultValue);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(staticDefaultValue);
    }

    @Override
    public String toString() {
      return "NonDynamicPojo{staticDefaultValue: " + staticDefaultValue + "}";
    }
  }

  public static class PojoUsingSdkApiAndLegacyApi {

    public PojoUsingSdkApiAndLegacyApi() {}

    @Parameter
    @Optional
    @NullSafe
    private ASimplePojo parameter1;

    @Parameter
    @Optional
    @org.mule.sdk.api.annotation.param.NullSafe
    private ASimplePojo parameter2;

    public void setParameters(ASimplePojo parameter1, ASimplePojo parameter2) {
      this.parameter1 = parameter1;
      this.parameter2 = parameter2;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof PojoUsingSdkApiAndLegacyApi) {
        PojoUsingSdkApiAndLegacyApi that = (PojoUsingSdkApiAndLegacyApi) o;
        return Objects.equals(parameter1, that.parameter1) && Objects.equals(parameter2, that.parameter2);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(parameter1, parameter2);
    }
  }

  public static class ASimplePojo {

    public String parameter = "parameter";

    public ASimplePojo() {}

    @Override
    public boolean equals(Object o) {
      if (o instanceof ASimplePojo) {
        ASimplePojo that = (ASimplePojo) o;
        return Objects.equals(this.parameter, that.parameter);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(parameter);
    }
  }
}

==> mule-module-extensions-support-4.11.0-20251110.184507-153-test-sources.jar: org/mule/runtime/module/extension/internal/runtime/resolver/NullSafeValueResolverWrapperTestCase.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.runtime.resolver;

import static org.mule.runtime.api.el.BindingContextUtils.getTargetBindingContext;
import static org.mule.runtime.api.message.Message.of;
import static org.mule.test.module.extension.internal.util.ExtensionsTestUtils.toMetadataType;

import static java.nio.charset.StandardCharsets.UTF_8;
import static java.util.Optional.empty;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.Is.is;

import static org.mockito.Answers.RETURNS_DEEP_STUBS;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mock.Strictness.LENIENT;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import org.mule.metadata.api.model.MetadataType;
import org.mule.runtime.api.config.ArtifactEncoding;
import org.mule.runtime.api.el.CompiledExpression;
import org.mule.runtime.api.exception.MuleException;
import org.mule.runtime.api.message.Message;
import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.api.transformation.TransformationService;
import org.mule.runtime.core.api.Injector;
import org.mule.runtime.core.api.el.ExpressionManagerSession;
import org.mule.runtime.core.api.el.ExtendedExpressionManager;
import org.mule.runtime.core.api.event.CoreEvent;
import org.mule.runtime.extension.api.annotation.param.NullSafe;
import org.mule.runtime.extension.api.annotation.param.Optional;
import org.mule.runtime.extension.api.annotation.param.Parameter;
import org.mule.runtime.extension.api.runtime.parameter.Literal;
import org.mule.runtime.module.extension.api.runtime.resolver.ValueResolver;
import org.mule.runtime.module.extension.api.runtime.resolver.ValueResolvingContext;
import org.mule.runtime.module.extension.internal.util.ReflectionCache;
import org.mule.tck.junit4.AbstractMuleTestCase;
import org.mule.tck.junit5.InjectDataWeave;
import org.mule.tck.size.SmallTest;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import io.qameta.allure.Issue;

@SmallTest
@ExtendWith(MockitoExtension.class)
class NullSafeValueResolverWrapperTestCase extends AbstractMuleTestCase {

  @Mock(answer = RETURNS_DEEP_STUBS, strictness = LENIENT)
  private CoreEvent event;

  @Mock
  private ObjectTypeParametersResolver objectTypeParametersResolver;

  @Mock
  private Injector injector;

  @Mock
  private TransformationService transformationService;

  @InjectDataWeave
  private ExtendedExpressionManager expressionManager;

  private ArtifactEncoding artifactEncoding = () -> UTF_8;

  private final ReflectionCache reflectionCache = new ReflectionCache();

  @BeforeEach
  public void setUp() throws MuleException {
    when(event.getError()).thenReturn(empty());
    when(event.getAuthentication()).thenReturn(empty());
    Message msg = of(null);
    when(event.getMessage()).thenReturn(msg);
    when(event.asBindingContext()).thenReturn(getTargetBindingContext(msg));
    when(event.getItemSequenceInfo()).thenReturn(empty());
  }

  @Test
  void testMapType() throws MuleException {
    assertExpected(new StaticValueResolver<>(null), toMetadataType(HashMap.class), false, new HashMap<>());
  }

  @Test
  void testPojoType() throws MuleException {
    ExpressionManagerSession session = mock(ExpressionManagerSession.class);
    when(session.evaluate(any(CompiledExpression.class), any(DataType.class)))
        .thenAnswer(inv -> new TypedValue<>(5, inv.getArgument(1)));
    expressionManager = spy(expressionManager);
    when(expressionManager.openSession(any())).thenReturn(session);

    assertExpected(new StaticValueResolver<>(null), toMetadataType(DynamicPojo.class), true, new DynamicPojo(5));

    verify(event, times(1)).asBindingContext();
  }

  @Test
  @Issue("W-14954976")
  void testPojoTypeWithLiteralExpression() throws MuleException {
    assertExpected(new StaticValueResolver<>(null),
                   toMetadataType(DynamicPojoWithLiteralParam.class),
                   false,
                   new DynamicPojoWithLiteralParam(new ImmutableLiteral<>("#[5]", Integer.class)));

    verify(event, times(1)).asBindingContext();
  }

  @Test
  void testPojoWithStaticDefaultValue() throws MuleException {
    when(transformationService.transform(eq("false"), any(), any())).thenReturn(false);
    assertExpected(new StaticValueResolver<>(null), toMetadataType(NonDynamicPojo.class), false, new NonDynamicPojo(false));
  }

  @Test
  void testPojoWithMap() throws MuleException {
    DynamicPojoWithMap pojo = new DynamicPojoWithMap();
    pojo.setMap(new HashMap<>());
    assertExpected(new StaticValueResolver<>(null), toMetadataType(DynamicPojoWithMap.class), false, pojo);
  }

  @Test
  void testNullSafeSdkAndLegacyAnnotation() throws MuleException {
    PojoUsingSdkApiAndLegacyApi pojo = new PojoUsingSdkApiAndLegacyApi();
    pojo.setParameters(new ASimplePojo(), new ASimplePojo());
    assertExpected(new StaticValueResolver<>(null), toMetadataType(PojoUsingSdkApiAndLegacyApi.class), false, pojo);
  }

  private void assertExpected(ValueResolver valueResolver, MetadataType type, boolean isDynamic, Object expected)
      throws MuleException {
    ValueResolver resolver = NullSafeValueResolverWrapper.of(valueResolver, type, reflectionCache, expressionManager,
                                                             artifactEncoding, injector, transformationService,
                                                             objectTypeParametersResolver);
    ValueResolvingContext ctx = ValueResolvingContext.builder(event)
        .withExpressionManager(expressionManager)
        .build();
    assertThat(resolver.isDynamic(), is(isDynamic));
    assertThat(resolver.resolve(ctx), is(expected));
  }

  public static class DynamicPojo {

    public DynamicPojo() {}

    public DynamicPojo(int time) {
      this.time = time;
    }

    @Parameter
    @Optional(defaultValue = "#[5]")
    private int time;

    public int getTime() {
      return time;
    }

    public void setTime(int time) {
      this.time = time;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojo) {
        DynamicPojo that = (DynamicPojo) o;
        return time == that.time;
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(time);
    }
  }


  public static class DynamicPojoWithLiteralParam {

    public DynamicPojoWithLiteralParam() {}

    public DynamicPojoWithLiteralParam(Literal<Integer> time) {
      this.time = time;
    }

    @Parameter
    @Optional(defaultValue = "#[5]")
    private Literal<Integer> time;

    public Literal<Integer> getTime() {
      return time;
    }

    public void setTime(Literal<Integer> time) {
      this.time = time;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojoWithLiteralParam) {
        DynamicPojoWithLiteralParam that = (DynamicPojoWithLiteralParam) o;
        return time.getLiteralValue().equals(that.time.getLiteralValue());
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(time.getLiteralValue());
    }
  }


  public static class DynamicPojoWithMap {

    public DynamicPojoWithMap() {}

    @Parameter
    @Optional
    @NullSafe
    private Map<String, String> map;

    public Map<String, String> getMap() {
      return map;
    }

    public void setMap(Map<String, String> map) {
      this.map = map;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof DynamicPojoWithMap) {
        DynamicPojoWithMap that = (DynamicPojoWithMap) o;
        return Objects.equals(map, that.map);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(map);
    }
  }


  public static class NonDynamicPojo {

    public NonDynamicPojo() {}

    public NonDynamicPojo(Boolean staticDefaultValue) {
      this.staticDefaultValue = staticDefaultValue;
    }

    @Parameter
    @Optional(defaultValue = "false")
    private Boolean staticDefaultValue;

    public Boolean getStaticDefaultValue() {
      return staticDefaultValue;
    }

    public void setStaticDefaultValue(Boolean staticDefaultValue) {
      this.staticDefaultValue = staticDefaultValue;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof NonDynamicPojo) {
        NonDynamicPojo that = (NonDynamicPojo) o;
        return Objects.equals(staticDefaultValue, that.staticDefaultValue);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(staticDefaultValue);
    }

    @Override
    public String toString() {
      return "NonDynamicPojo{staticDefaultValue: " + staticDefaultValue + "}";
    }
  }

  public static class PojoUsingSdkApiAndLegacyApi {

    public PojoUsingSdkApiAndLegacyApi() {}

    @Parameter
    @Optional
    @NullSafe
    private ASimplePojo parameter1;

    @Parameter
    @Optional
    @org.mule.sdk.api.annotation.param.NullSafe
    private ASimplePojo parameter2;

    public void setParameters(ASimplePojo parameter1, ASimplePojo parameter2) {
      this.parameter1 = parameter1;
      this.parameter2 = parameter2;
    }

    @Override
    public boolean equals(Object o) {
      if (o instanceof PojoUsingSdkApiAndLegacyApi) {
        PojoUsingSdkApiAndLegacyApi that = (PojoUsingSdkApiAndLegacyApi) o;
        return Objects.equals(parameter1, that.parameter1) && Objects.equals(parameter2, that.parameter2);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(parameter1, parameter2);
    }
  }

  public static class ASimplePojo {

    public String parameter = "parameter";

    public ASimplePojo() {}

    @Override
    public boolean equals(Object o) {
      if (o instanceof ASimplePojo) {
        ASimplePojo that = (ASimplePojo) o;
        return Objects.equals(this.parameter, that.parameter);
      }
      return false;
    }

    @Override
    public int hashCode() {
      return Objects.hash(parameter);
    }
  }
}
